--Core of addon
local testData, core = ...;
MorthalinRaidTools = LibStub("AceAddon-3.0"):NewAddon(core, "Morthalin Raid Tools", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "AceSerializer-3.0")
local MorthalinRaidTools	= LibStub("AceAddon-3.0"):GetAddon("Morthalin Raid Tools")
local AceGUI = LibStub("AceGUI-3.0")
local LibDeflate = LibStub:GetLibrary("LibDeflate")

--COMMAND options
local options = {
	name = 'MorthalinRaidTools',
	type = 'group',
	args = {
		bench = {
			type = 'input',
			name = "Add member",
			desc = "Add guild member on bench /mrt bench playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:AddOnBench(playerName, playerName); end
		},
		rm = {
			type = 'input',
			name = "Remove member",
			desc = "Remove member from bench /mrt rm playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:RemoveFromBench(playerName); end
		},
		list = {
			type = 'execute',
			name = "Show players",
			desc = "Show acutal list of players on bench",
			func = function() MorthalinRaidTools:GetList(); end
		},
		history = {
			type = 'execute',
			name = "Show history",
			desc = "Show history list",
			func = function() MorthalinRaidTools:ShowHistory(); end
		},
		reset = {
			type = 'execute',
			name = "Empty bench",
			desc = "Clear list of players on bench",
			func = function() MorthalinRaidTools:ResetList(); end
		},
		resetHistory = {
			type = 'execute',
			name = "Empty history",
			desc = "Clear list of kills",
			func = function() MorthalinRaidTools:ResetHistory(); end
		},
		resetGamble = {
			type = 'execute',
			name = "Reset gamble statistics",
			desc = "Clear gamble statistics",
			func = function() MorthalinRaidTools:ClearGambleStatistics(); end
		},
		sendBuffs = {
			type = 'input',
			name = "Send buff settings to player",
			desc = "Send buff settings to player /mrt sendBuffs playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:SendBuffSettings(playerName); end
		},
	}
}

local settings = {
	name = 'Morthalin Raid Tools Settings',
	type = 'group',
	args = {
		broadcastRequestHeader = {
			type = "header",
			name = "Broadcast requests",
			order = 1,
		},
		broadcastRequestBench = {
			order = 2,
			type = "execute",
			name = "Request bench",
			desc = "Request for broadcast from main broadcaster players on bench.",
			func = function() MorthalinRaidTools:AskForBenchBroadcast(); end,
		},
		broadcastRequest7History = {
			order = 3,
			type = "execute",
			name = "Request history 7D",
			desc = "Request for broadcast from main broadcaster all kills of last 7 days.",
			func = function() MorthalinRaidTools:AskForHistory7Broadcast(); end,
		},
		broadcastHeader = {
			type = "header",
			name = "Broadcast options",
			order = 4,
		},
		mainBroadcaster = {
			order = 5,
			type = "toggle",
			name = "Main broadcaster",
			desc = "Set self as broadcaster for required bench and history broadcasts. Also only main broadcaster whisper to player moved from bench to raid.",
			get = function() return MorthalinRaidTools.db.global.mainBroadcaster; end,
			set = function(info, v) MorthalinRaidTools.db.global.mainBroadcaster = v; end,
		},
		timeStampFormat = {
			type = "select",
			name = "Allow broadcast from rank",
			desc = "Set rank which sender need to accept his broadcast.",
			values = function()
				local ranks = {}
				for i = 1, GuildControlGetNumRanks() do
					table.insert(ranks, GuildControlGetRankName(i));
				end
				return ranks;
			end,
			order = 6,
			get = function() return MorthalinRaidTools.db.global.broadcastIncomingRank; end,
			set = function(info, v) MorthalinRaidTools.db.global.broadcastIncomingRank = v; end,
		},
		broadcastAll = {
			order = 7,
			type = "execute",
			name = "Broadcast history",
			desc = "Broadcast full history of kills to all guild members. For broadcast only one kill, instance or date use broadcast at History.",
			func = function() MorthalinRaidTools:BroadcastHistory(); end,
		},
		broadcastReceive = {
			order = 8,
			type = "toggle",
			name = "Allow broadcasts receive",
			desc = "Allow receiving broadcasts from guild members.",
			width = 1.2,
			get = function() return MorthalinRaidTools.db.global.allowBroadcastIn; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcastIn = v; end,
		},
		broadcastSend = {
			order = 9,
			type = "toggle",
			name = "Allow broadcasts sending",
			desc = "Allow sending broadcasts to guild members.",
			width = 1.2,
			get = function() return MorthalinRaidTools.db.global.allowBroadcastOut; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcastOut = v; end,
		},
		broadcastRemove = {
			order = 10,
			type = "toggle",
			name = "Allow remove",
			desc = "Allow receiving broadcasts for remove players or whole kills from history.",
			get = function() return MorthalinRaidTools.db.global.allowBroadcastRemove; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcastRemove = v; end,
		},
		broadcastChange = {
			order = 11,
			type = "toggle",
			name = "Allow change",
			desc = "Allow receiving broadcasts to change players at kills in history.",
			get = function() return MorthalinRaidTools.db.global.allowBroadcastChange; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcastChange = v; end,
		},
		broadcastOnlyGuild = {
			order = 12,
			type = "toggle",
			name = "Only guild raids",
			desc = "Allow sending broadcasts only kills where was at least 50% of guild members.",
			get = function() return MorthalinRaidTools.db.global.allowOnlyGuildBroadcast; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowOnlyGuildBroadcast = v; end,
		},
		historyHeader = {
			type = "header",
			name = "History options",
			order = 13,
		},
		historyTooltip = {
			order = 14,
			type = "toggle",
			name = "AutoHide Tooltip",
			desc = "Automatically hide tooltips on history view.",
			get = function() return MorthalinRaidTools.db.global.autoHideTooltip; end,
			set = function(info, v) MorthalinRaidTools.db.global.autoHideTooltip = v; end,
		},
		historyTooltipTime = {
			order = 15,
			type = "input",
			name = "AutoHide Time",
			desc = "Time in seconds after tooltip hide in history.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.autoHideTooltipTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.autoHideTooltipTime = tonumber(v); end end,
		},
		historyOnlyGuild = {
			order = 16,
			type = "toggle",
			name = "Only guild raids",
			desc = "Allow save only raids where was at least 50% of guild members.",
			get = function() return MorthalinRaidTools.db.global.allowOnlyGuildKills; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowOnlyGuildKills = v; end,
		},
		historyHistorySettings = {
			order = 17,
			type = "execute",
			name = "History Settings",
			desc = "Settings for manage/create kills in history",
			func = function() MorthalinRaidTools:ShowHistorySettings() end,
		},
		historyBuffsSettings = {
			order = 18,
			type = "execute",
			name = "Buffs Settings",
			desc = "Settings of visible buffs in history",
			func = function() MorthalinRaidTools:ConstructBuffsSettings(); end,
		},
		snapshotHeader = {
			type = "header",
			name = "Buff snapshot options",
			order = 19,
		},
		snapshotTooltip = {
			order = 20,
			type = "toggle",
			name = "AutoHide Tooltip",
			desc = "Automatically hide tooltips on snapshot button.",
			get = function() return MorthalinRaidTools.db.global.autoHideTooltipSnapshot; end,
			set = function(info, v) MorthalinRaidTools.db.global.autoHideTooltipSnapshot = v; end,
		},
		snapshotTooltipTime = {
			order = 21,
			type = "input",
			name = "AutoHide Time",
			desc = "Time in seconds after tooltip hide on snapshot button.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.autoHideTooltipSnapshotTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.autoHideTooltipSnapshotTime = tonumber(v); end end,
		},
		benchHeader = {
			type = "header",
			name = "Bench options",
			order = 22,
		},
		autoBench = {
			order = 23,
			type = "toggle",
			name = "Auto bench invites",
			desc = "This option will automatically allow bench invites after raid formed by whisper !bench or !bench 'nickname' if it was disabled before.",
			get = function() return MorthalinRaidTools.db.global.checkBenchAuto; end,
			set = function(info, v) MorthalinRaidTools.db.global.checkBenchAuto = v; end,
		},
		askBenchDisable = {
			order = 24,
			type = "toggle",
			name = "Ask for disable bench",
			desc = "This option will automatically ask for disable bench after raid leaved.",
			get = function() return MorthalinRaidTools.db.global.askBenchDis; end,
			set = function(info, v) MorthalinRaidTools.db.global.askBenchDis = v; end,
		},
		autoBenchEmpty= {
			order = 25,
			type = "toggle",
			name = "Ask for empty bench",
			desc = "This option will automatically ask for empty bench after raid leaved.",
			get = function() return MorthalinRaidTools.db.global.autoBenchEmpty; end,
			set = function(info, v) MorthalinRaidTools.db.global.autoBenchEmpty = v; end,
		},
		mainBenchInv= {
			order = 26,
			type = "toggle",
			name = "Check for main",
			desc = "Check for main and add it on bench instead.",
			get = function() return MorthalinRaidTools.db.global.mainBenchInv; end,
			set = function(info, v) MorthalinRaidTools.db.global.mainBenchInv = v; end,
		},
		statisticsHeader = {
			type = "header",
			name = "Statistics options",
			order = 27,
		},
		mainRaids = {
			type = "multiselect",
			name = "Main raids",
			desc = "Set main raids to count attendance on.",
			values = function()
				local uniqueZones = {};
				local zones = {};
				for i = 1, #MorthalinRaidTools.db.global.history do
					if(uniqueZones[MorthalinRaidTools.db.global.history[i].zone] == nil) then
						uniqueZones[MorthalinRaidTools.db.global.history[i].zone] = {zone = MorthalinRaidTools.db.global.history[i].zone};
					end
				end
				
				for index, value in pairs(uniqueZones) do
					table.insert(zones, index);
				end
				local function sortZones(a, b)
					return a < b;
				end
				table.sort(zones, sortZones);
				
				if(#MorthalinRaidTools.db.global.mainRaids < #zones) then
					for i = #MorthalinRaidTools.db.global.mainRaids + 1, #zones do
						table.insert(MorthalinRaidTools.db.global.mainRaids, {zone = zones[i], value = true});
					end
				end
				return zones;
			end,
			order = 28,
			get = function(info, index) return MorthalinRaidTools.db.global.mainRaids[index].value; end,
			set = function(info, index, value) MorthalinRaidTools.db.global.mainRaids[index].value = value; end,
		},
		itemStatsHeader = {
			type = "header",
			name = "Item Statistics options",
			order = 29,
		},
		itemStatsSettings = {
			order = 30,
			type = "execute",
			name = "Item Settings",
			desc = "Settings of item to be counted in item statistics",
			func = function() MorthalinRaidTools:ConstructItemSettings(); end,
		},
		itemStatsImport = {
			order = 31,
			type = "execute",
			name = "Import",
			desc = "Import of item settings",
			func = function() MorthalinRaidTools:ShowEditWindow(true, 5); end,
		},
		itemStatsBroadcast = {
			order = 32,
			type = "execute",
			name = "Broadcast",
			desc = "Broadcast all data from item settings to guild members",
			func = function() MorthalinRaidTools:SendItemSettings(); end,
		},
		itemStatsOnlyGuild = {
			order = 33,
			type = "toggle",
			name = "Only guild raids",
			desc = "Allow save only raids where was at least 50% of guild members.",
			get = function() return MorthalinRaidTools.db.global.checkOnlyGuildItems; end,
			set = function(info, v) MorthalinRaidTools.db.global.checkOnlyGuildItems = v; end,
		},
		itemRollHeader = {
			type = "header",
			name = "Item Rolling options",
			order = 34,
		},
		itemRollRange = {
			order = 35,
			type = "input",
			name = "Roll number",
			desc = "Check if players use /roll with selecter number. For /roll without number is default value 100.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.itemRollRange); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.itemRollRange = tonumber(v); end end,
		},
		itemRollPhrase = {
			order = 36,
			type = "input",
			name = "Roll phrase",
			desc = "Phrase which reset stored rolls. Use <item> for item links. Only Master looter can write reset phrase!",
			get = function() return MorthalinRaidTools.db.global.itemRollResetPhrase; end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.itemRollResetPhrase = v; end end,
		},
		itemRollResetLeader = {
			order = 37,
			type = "toggle",
			name = "Leader phrase",
			desc = "Allow raid leader to reset rolls with phrase if it's not same player as master looter.",
			get = function() return MorthalinRaidTools.db.global.itemRollResetLeader; end,
			set = function(info, v) MorthalinRaidTools.db.global.itemRollResetLeader = v; end,
		},
		itemRollOpenStats = {
			order = 38,
			type = "toggle",
			name = "Open Statistics",
			desc = "Open item statistics after roll phrase called by master looter.",
			get = function() return MorthalinRaidTools.db.global.itemRollOpenStats; end,
			set = function(info, v) MorthalinRaidTools.db.global.itemRollOpenStats = v; end,
		},
		itemArchHeader = {
			type = "header",
			name = "Item Statistics archive",
			order = 39,
		},
		maxArch = {
			order = 40,
			type = "input",
			name = "Max count",
			desc = "Maximum count of archived data. Start on first login each day.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.archiveItemCountersN); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.archiveItemCountersN = tonumber(v); end end,
		},
		archSelects = {
			type = "select",
			name = "Pick archive",
			desc = "Pick date of archive to restore.",
			values = function()
				local dates = {}
				for i = 1, #MorthalinRaidTools.db.global.archiveItemCounters do
					table.insert(dates, MorthalinRaidTools.db.global.archiveItemCounters[i].archDate);
				end
				return dates;
			end,
			order = 41,
			get = function() MorthalinRaidTools.selectedArchiveIndex = 1; return 1; end,
			set = function(info, v) MorthalinRaidTools.selectedArchiveIndex = v; end,
		},
		restoreItemData = {
			order = 42,
			type = "execute",
			name = "Restore",
			desc = "Restore selected item data",
			func = function() MorthalinRaidTools:RestoreItemData(MorthalinRaidTools.selectedArchiveIndex); end,
		},
		exportHeader = {
			type = "header",
			name = "Export options",
			order = 43,
		},
		expShowBosses = {
			order = 44,
			type = "toggle",
			name = "Show Bosses",
			desc = "Export Boss columns at date and instance exports to see attendance at them.",
			get = function() return MorthalinRaidTools.db.global.expBosses; end,
			set = function(info, v) MorthalinRaidTools.db.global.expBosses = v; end,
		},
		expShowInstance = {
			order = 45,
			type = "toggle",
			name = "Show Instance",
			desc = "Show Instance columns at date export to see attendance for every instance.",
			get = function() return MorthalinRaidTools.db.global.expInstance; end,
			set = function(info, v) MorthalinRaidTools.db.global.expInstance = v; end,
		},
		expShowAll = {
			order = 46,
			type = "toggle",
			name = "Show All",
			desc = "Show All column at date and instance export to see attendance for whole day or instance.",
			get = function() return MorthalinRaidTools.db.global.expAll; end,
			set = function(info, v) MorthalinRaidTools.db.global.expAll = v; end,
		},
		expMergeNoId = {
			order = 47,
			type = "toggle",
			name = "Show only one buff at setting",
			desc = "Show only one buff if players used more of same name and spell ID is empty in settings.",
			width = 1.3,
			get = function() return MorthalinRaidTools.db.global.expBuffsMerge; end,
			set = function(info, v) MorthalinRaidTools.db.global.expBuffsMerge = v; end,
		},
		expBuffsMains = {
			order = 48,
			type = "toggle",
			name = "Show Mains",
			desc = "Show main characters instead of alts.",
			width = 1.3,
			get = function() return MorthalinRaidTools.db.global.expBuffsMains; end,
			set = function(info, v) MorthalinRaidTools.db.global.expBuffsMains = v; end,
		},
		importHeader = {
			type = "header",
			name = "Import options",
			order = 49,
		},
		impMaxLvlAlts = {
			order = 50,
			type = "toggle",
			name = "Find only full lvl Alts",
			desc = "On schedule import find only full level alts.",
			width = 1.3,
			get = function() return MorthalinRaidTools.db.global.impMaxLvlAlts; end,
			set = function(info, v) MorthalinRaidTools.db.global.impMaxLvlAlts = v; end,
		},
		gambleHeader = {
			type = "header",
			name = "Gamble options",
			order = 51,
		},
		autoGamble = {
			order = 52,
			type = "toggle",
			name = "Auto game timers",
			desc = "This option will automatically process game with set timers.",
			get = function() return MorthalinRaidTools.db.global.gambleAuto; end,
			set = function(info, v) MorthalinRaidTools.db.global.gambleAuto = v; end,
		},
		autoJoinGamble = {
			order = 53,
			type = "toggle",
			name = "Auto join",
			desc = "This option will automatically join and roll.",
			get = function() return MorthalinRaidTools.db.global.gambleJoin; end,
			set = function(info, v) MorthalinRaidTools.db.global.gambleJoin = v; end,
		},
		inviteTimer = {
			order = 54,
			type = "input",
			name = "Invite time",
			desc = "Time interval before last call.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleInviteTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleInviteTime = tonumber(v); end end,
		},
		lastCallTimer = {
			order = 55,
			type = "input",
			name = "Last call time",
			desc = "Time interval before start rolling.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleLastCallTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleLastCallTime = tonumber(v); end end,
		},
		ReminderTimer = {
			order = 56,
			type = "input",
			name = "Reminder",
			desc = "Time interval to remind players not rolled.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleRollReminder); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleRollReminder = tonumber(v); end end,
		},
	}
}

--SCHEME FOR DB
local defaults = {
	global = {
		bench = {
			players = {},
		},
		version = 0,
		acceptBench = false,
		allowBroadcastIn = true,
		allowBroadcastOut = false,
		allowBroadcastRemove = false,
		allowBroadcastChange = true,
		allowOnlyGuildBroadcast = true,
		allowOnlyGuildKills = true,
		mainBroadcaster = false,
		broadcastIncomingRank = 1,
		mainRaids = {},
		autoHideTooltip = true,
		autoHideTooltipTime = 0,
		autoHideTooltipSnapshot = true,
		autoHideTooltipSnapshotTime = 0,
		checkBenchAuto = false,
		askBenchDis = false,
		autoBenchEmpty = true,
		mainBenchInv = true,
		minimap = {hide = false},
		history = {},
		members = {},
		bossfight = false,
		uniqueBuffs = {},
		showBuffs = true,
		showBuffsAll = true,
		buffs = {},
		buffsSnapshotTime = nil,
		enchants = {},
		schedule = {},
		itemCounter = {},
		itemRollsCounter = {},
		itemRollRange = 100,
		itemRollResetPhrase = "<item>ROLL",
		itemRollResetLeader = true,
		itemRollOpenStats = true,
		checkOnlyGuildItems = true,
		archiveItemCounters = {},
		archiveItemCountersN = 3,
		gambleStatistics = {},
		gambleAuto = false,
		gambleJoin = false,
		gambleInviteTime = 10,
		gambleLastCallTime = 3,
		gambleRollReminder = 5,
		expAll = true,
		expInstance = true,
		expBosses = true,
		expBuffsMerge = true,
		expBuffsMains = true,
		impMaxLvlAlts = true,
		encounterList = {
			["Lucifron"] = {
				id = 663,
				name = "Lucifron",
				instance = "Molten Core",
			},
			["Magmadar"] = {
				id = 664,
				name = "Magmadar",
				instance = "Molten Core",
			},
			["Gehennas"] = {
				id = 665,
				name = "Gehennas",
				instance = "Molten Core",
			},
			["Garr"] = {
				id = 666,
				name = "Garr",
				instance = "Molten Core",
			},
			["Shazzrah"] = {
				id = 667,
				name = "Shazzrah",
				instance = "Molten Core",
			},
			["Baron Geddon"] = {
				id = 668,
				name = "Baron Geddon",
				instance = "Molten Core",
			},
			["Sulfuron Harbinger"] = {
				id = 669,
				name = "Sulfuron Harbinger",
				instance = "Molten Core",
			},
			["Golemagg the Incinerator"] = {
				id = 670,
				name = "Golemagg the Incinerator",
				instance = "Molten Core",
			},
			["Majordomo Executus"] = {
				id = 671,
				name = "Majordomo Executus",
				instance = "Molten Core",
			},
			["Ragnaros"] = {
				id = 672,
				name = "Ragnaros",
				instance = "Molten Core",
			},
			["Razorgore the Untamed"] = {
				id = 610,
				name = "Razorgore the Untamed",
				instance = "Blackwing Lair",
			},
			["Vaelastrasz the Corrupt"] = {
				id = 611,
				name = "Vaelastrasz the Corrupt",
				instance = "Blackwing Lair",
			},
			["Broodlord Lashlayer"] = {
				id = 612,
				name = "Broodlord Lashlayer",
				instance = "Blackwing Lair",
			},
			["Firemaw"] = {
				id = 613,
				name = "Firemaw",
				instance = "Blackwing Lair",
			},
			["Ebonroc"] = {
				id = 614,
				name = "Ebonroc",
				instance = "Blackwing Lair",
			},
			["Flamegor"] = {
				id = 615,
				name = "Flamegor",
				instance = "Blackwing Lair",
			},
			["Chromaggus"] = {
				id = 616,
				name = "Chromaggus",
				instance = "Blackwing Lair",
			},
			["Nefarian"] = {
				id = 617,
				name = "Nefarian",
				instance = "Blackwing Lair",
			},
			["Onyxia"] = {
				id = 1084,
				name = "Onyxia",
				instance = "Onyxia's Lair",
			},
			["High Priest Venoxis"] = {
				id = 784,
				name = "High Priest Venoxis",
				instance = "Zul'Gurub",
			},
			["High Priestess Jeklik"] = {
				id = 785,
				name = "High Priestess Jeklik",
				instance = "Zul'Gurub",
			},
			["High Priestess Mar'li"] = {
				id = 786,
				name = "High Priestess Mar'li",
				instance = "Zul'Gurub",
			},
			["Bloodlord Mandokir"] = {
				id = 787,
				name = "Bloodlord Mandokir",
				instance = "Zul'Gurub",
			},
			["Edge of Madness"] = {
				id = 788,
				name = "Edge of Madness",
				instance = "Zul'Gurub",
			},
			["High Priest Thekal"] = {
				id = 789,
				name = "High Priest Thekal",
				instance = "Zul'Gurub",
			},
			["Gahz'ranka"] = {
				id = 790,
				name = "Gahz'ranka",
				instance = "Zul'Gurub",
			},
			["High Priestess Arlokk"] = {
				id = 791,
				name = "High Priestess Arlokk",
				instance = "Zul'Gurub",
			},
			["Jin'do the Hexxer"] = {
				id = 792,
				name = "Jin'do the Hexxer",
				instance = "Zul'Gurub",
			},
			["Hakkar"] = {
				id = 793,
				name = "Hakkar",
				instance = "Zul'Gurub",
			},
			["The Prophet Skeram"] = {
				id = 709,
				name = "The Prophet Skeram",
				instance = "Ahn'Qiraj Temple",
			},
			["Silithid Royalty"] = {
				id = 710,
				name = "Silithid Royalty",
				instance = "Ahn'Qiraj Temple",
			},
			["Battleguard Sartura"] = {
				id = 711,
				name = "Battleguard Sartura",
				instance = "Ahn'Qiraj Temple",
			},
			["Fankriss the Unyielding"] = {
				id = 712,
				name = "Fankriss the Unyielding",
				instance = "Ahn'Qiraj Temple",
			},
			["Viscidus"] = {
				id = 713,
				name = "Viscidus",
				instance = "Ahn'Qiraj Temple",
			},
			["Princess Huhuran"] = {
				id = 714,
				name = "Princess Huhuran",
				instance = "Ahn'Qiraj Temple",
			},
			["Twin Emperors"] = {
				id = 715,
				name = "Twin Emperors",
				instance = "Ahn'Qiraj Temple",
			},
			["Ouro"] = {
				id = 716,
				name = "Ouro",
				instance = "Ahn'Qiraj Temple",
			},
			["C'thun"] = {
				id = 717,
				name = "C'thun",
				instance = "Ahn'Qiraj Temple",
			},
			["Kurinnaxx"] = {
				id = 718,
				name = "Kurinnaxx",
				instance = "Ruins of Ahn'Qiraj",
			},
			["General Rajaxx"] = {
				id = 719,
				name = "General Rajaxx",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Moam"] = {
				id = 720,
				name = "Moam",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Buru the Gorger"] = {
				id = 721,
				name = "Buru the Gorger",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Ayamiss the Hunter"] = {
				id = 722,
				name = "Ayamiss the Hunter",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Ossirian the Unscarred"] = {
				id = 723,
				name = "Ossirian the Unscarred",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Anub'Rekhan"] = {
				id = 1107,
				name = "Anub'Rekhan",
				instance = "Naxxramas",
			},
			["Gluth"] = {
				id = 1108,
				name = "Gluth",
				instance = "Naxxramas",
			},
			["Gothik the Harvester"] = {
				id = 1109,
				name = "Gothik the Harvester",
				instance = "Naxxramas",
			},
			["Grand Widow Faerlina"] = {
				id = 1110,
				name = "Grand Widow Faerlina",
				instance = "Naxxramas",
			},
			["Grobbulus"] = {
				id = 1111,
				name = "Grobbulus",
				instance = "Naxxramas",
			},
			["Heigan the Unclean"] = {
				id = 1112,
				name = "Heigan the Unclean",
				instance = "Naxxramas",
			},
			["Instructor Razuvious"] = {
				id = 1113,
				name = "Instructor Razuvious",
				instance = "Naxxramas",
			},
			["Kel'Thuzad"] = {
				id = 1114,
				name = "Kel'Thuzad",
				instance = "Naxxramas",
			},
			["Loatheb"] = {
				id = 1115,
				name = "Loatheb",
				instance = "Naxxramas",
			},
			["Maexxna"] = {
				id = 1116,
				name = "Maexxna",
				instance = "Naxxramas",
			},
			["Noth the Plaguebringer"] = {
				id = 1117,
				name = "Noth the Plaguebringer",
				instance = "Naxxramas",
			},
			["Patchwerk"] = {
				id = 1118,
				name = "Patchwerk",
				instance = "Naxxramas",
			},
			["Sapphiron"] = {
				id = 1119,
				name = "Sapphiron",
				instance = "Naxxramas",
			},
			["Thaddius"] = {
				id = 1120,
				name = "Thaddius",
				instance = "Naxxramas",
			},
			["The Four Horsemen"] = {
				id = 1121,
				name = "The Four Horsemen",
				instance = "Naxxramas",
			}
		},
		instances = {
			["Molten Core"] = "MC",
			["Blackwing Lair"] = "BWL",
			["Onyxia's Lair"] = "Onyx",
			["Zul'Gurub"] = "ZG",
			["Ahn'Qiraj Temple"] = "AQ40",
			["Ruins of Ahn'Qiraj"] = "AQ20",
			["Naxxramas"] = "NAXX"
		},
		enchantList = {
			["Brilliant Wizard Oil"] = 134727,
			["Brilliant Mana Oil"] = 134722,
			["Wizard Oil"] = 134726,
			["Lesser Mana Oil"] = 134879,
			["Sharpen Weapon - Critical"] = 135228,
			["Sharpen Blade V"] = 135252,
		},
		maxLevelOnExpansion = {
			[0] = 60,
			[1] = 70,
			[2] = 80,
			[3] = 85,
			[4] = 90,
			[5] = 100,
			[6] = 110,
			[7] = 120,
		},
		resets = {
			["ZG"] = 1587020400,
			["Onyx"] = 1586329200,
			["MC"] = 1586329200,
		}
	},
	char = {
		minimap = {hide = false},
		raid = IsInRaid(),
		party = IsInGroup(),
	}
}

--EVENTS
MorthalinRaidTools:RegisterEvent("BOSS_KILL", function(...) MorthalinRaidTools:BossKilled(...) end);

--Init function
function MorthalinRaidTools:OnInitialize()
	MorthalinRaidTools.db = LibStub("AceDB-3.0"):New("MorthalinRaidToolsDB", defaults);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MorthalinRaidTools", options, {"morthalinRaidTools", "mrt"});
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MorthalinRaidTools-Settings", settings);
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MorthalinRaidTools-Settings", "Morthalin Raid Tools");
	MorthalinRaidTools.debug = false;
	MorthalinRaidTools.versionWarrning = false;
	if(MorthalinRaidTools.db.global.acceptBench) then
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_GUILD", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_BN_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	end
	MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID_LEADER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID_WARNING", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	MorthalinRaidTools:RegisterEvent("CHAT_MSG_SYSTEM", function(...) MorthalinRaidTools:SystemChannelCallback(...); end);
	MorthalinRaidTools:RegisterEvent("GROUP_ROSTER_UPDATE", function(...) MorthalinRaidTools:PartyChanged(...) end);
	MorthalinRaidTools:RegisterEvent("ENCOUNTER_START", function(...) MorthalinRaidTools:BossPulled(...) end);
	MorthalinRaidTools:RegisterEvent("ENCOUNTER_END", function(...) MorthalinRaidTools:BossFightEnd(...) end);
	MorthalinRaidTools:RegisterEvent("PLAYER_ENTERING_WORLD", function(...) MorthalinRaidTools:ZoneChanged(...) end);
	MorthalinRaidTools:RegisterEvent("ZONE_CHANGED_NEW_AREA", function(...) MorthalinRaidTools:ZoneChanged(...) end);
	MorthalinRaidTools:RegisterEvent("PLAYER_REGEN_DISABLED", function(...) if(MorthalinRaidTools.ui.menuFrame:IsVisible()) then MorthalinRaidTools:ToggleMenu(); end end);
	MorthalinRaidTools:RegisterEvent("CHAT_MSG_LOOT", function(...) MorthalinRaidTools:LootMsg(...) end);
	MorthalinRaidTools:RegisterComm("MRTAddBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTRemoveBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTFullBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTEmptyBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTKill", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTHistory", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAddHistPlayer", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTRemoveKill", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAskForBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAskForHist7", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTBuffSettings", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTItemSettings", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTVersionCheck", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:CheckVersion();
	MorthalinRaidTools:ConstructUI();
	MorthalinRaidTools:ConstructMiniMapIcon();
	MorthalinRaidTools:RemoveExpiredEnchants();
	MorthalinRaidTools:ArchiveItemCouterData();
end

-- Callback function for chat invites to raid
-- =========================
function MorthalinRaidTools:ChatChannelCallback(...)
	local message = select(2, ...)
	local sender = select(3, ...)
	message = message:gsub("%s+", "") -- trim whitespace
	sender = Ambiguate(sender, "short")
	if (message == "!bench" and MorthalinRaidTools.db.global.acceptBench) then
		MorthalinRaidTools:AddOnBench(sender, sender);
	elseif (string.sub(message, 1, 6) == "!bench" and MorthalinRaidTools.db.global.acceptBench) then
		MorthalinRaidTools:AddOnBench(string.sub(message, 7, -1), sender);
	elseif (message == "1" and MorthalinRaidTools:AcceptInvGamble()) then
		MorthalinRaidTools:AddPlayerGamble(sender);
	elseif ((message == "inv" or message == "invite") and MorthalinRaidTools.autoInvSchedule) then
		MorthalinRaidTools:CheckLoggedPlayer(sender, true);
	end
	if(MorthalinRaidTools:CheckRollPhrase(sender, message)) then
		if(MorthalinRaidTools.db.global.itemRollOpenStats and MorthalinRaidTools.ui.itemStatFrame:IsVisible() == false) then
			MorthalinRaidTools:ToggleItemStat();
		end
		MorthalinRaidTools.db.global.itemRollsCounter = {};
		if(MorthalinRaidTools.ui.itemStatFrame:IsVisible()) then
			MorthalinRaidTools:SortItemStatByRolls();
		end
	end
end

-- Callback function for system chat
-- =========================
function MorthalinRaidTools:SystemChannelCallback(...)
	local message = select(2, ...);
	local player, word1, word2, word3 = strsplit(" ", message);
	if (word1 == "rolls" and MorthalinRaidTools:AcceptRollsGamble()) then
		MorthalinRaidTools:CallbackStartRolling(player, word2, word3);
	elseif(word1 == "rolls") then
		MorthalinRaidTools:PlayerRolledForItem(player, word2, word3);
	elseif (word1 == "has" and word2 == "come" and word3 == "online." and MorthalinRaidTools.autoInvSchedule) then
		name = player:sub(player:find(":[^%[]*%["));
		name = name:sub(2, -4);
		MorthalinRaidTools:CheckLoggedPlayer(name);
	end
end

-- Callback function for boss pull
-- =========================
function MorthalinRaidTools:BossPulled(...)
	local event = select(1, ...);
	local encounterID = select(2, ...);
	local encounterName = select(3, ...);
	self.db.global.bossfight = true;
	MorthalinRaidTools:CheckBuffs();
	MorthalinRaidTools:Print("Encounter started");
end

-- Callback function for boss fight end
-- =========================
function MorthalinRaidTools:BossFightEnd(...)
	local event = select(1, ...);
	local encounterID = select(2, ...);
	local encounterName = select(3, ...);
	self.db.global.bossfight = false;
	MorthalinRaidTools:Print("Encounter end");
end

-- Callback function for boss kill
-- =========================
function MorthalinRaidTools:BossKilled(...)
	local event = select(1, ...);
	local encounterID = select(2, ...);
	local encounterName = select(3, ...);
	MorthalinRaidTools:AddBossKill(encounterID, encounterName);
end

-- Callback function for after invited
-- =========================
function MorthalinRaidTools:PartyChanged(...)
	if(UnitInBattleground("player") == nil) then
		if(self.db.char.raid ~= IsInRaid() and IsInRaid()) then
			self:OnRaidCreated();
		elseif(self.db.char.raid ~= IsInRaid() and IsInGroup() == false) then
			self:OnRaidLeave();
		elseif(IsInRaid()) then
			self:CheckBenchInRaid();
		end
		
		if(self.db.char.party ~= IsInGroup() and IsInGroup() and MorthalinRaidTools.autoInvSchedule) then
			ConvertToRaid();
		end
		
		self.db.char.party = IsInGroup();
		self.db.char.raid = IsInRaid();
	end
end

-- Callback function for zone changed
-- =========================
function MorthalinRaidTools:ZoneChanged(...)
	if(MorthalinRaidTools.db.global.instances[GetInstanceInfo()] ~= nil) then
		MorthalinRaidTools:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", function(...) MorthalinRaidTools:SpellCasted(...) end);
	else
		MorthalinRaidTools:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end
end

function MorthalinRaidTools:LootMsg(...)
	local msg = select(2, ...);
	local player = select(6, ...);
	local playerInfo = MorthalinRaidTools:GetPlayerInfo(player);
	local word1, word2, word3 = strsplit(" ", msg);
	local guildMembers, raidMembers = 0, 0;
	local item = msg:sub(msg:find("|h[^|]+|h|r") + 3, -7);
	if((word2 == "receive" or word2 == "receives") and word3 == "loot:" and playerInfo ~= nil) then
		if(MorthalinRaidTools.db.global.itemCounter[item] ~= nil) then
		
			if(MorthalinRaidTools.db.global.checkOnlyGuildItems) then
				for i=1, 40 do
					local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
					if(name ~= nil) then
						if MorthalinRaidTools:IsGuildMember(name) then
							guildMembers = guildMembers + 1;
						end
						raidMembers = raidMembers + 1;
					end
				end
			end
		
			if(MorthalinRaidTools.db.global.checkOnlyGuildItems == false or ((guildMembers * 100 / raidMembers) >= 50)) then
				if(MorthalinRaidTools.db.global.itemCounter[item].players[player] == nil) then
					MorthalinRaidTools.db.global.itemCounter[item].players[player] = 1;
				else
					MorthalinRaidTools.db.global.itemCounter[item].players[player] = MorthalinRaidTools.db.global.itemCounter[item].players[player] + 1;
				end
			end
		end
	end
end

function MorthalinRaidTools:SpellCasted(...)
	local param1, event, param3, param4, caster, param6, param7, param8, param9, param10, param11, param12, spell = CombatLogGetCurrentEventInfo();
	if(event == "SPELL_CAST_SUCCESS" and (MorthalinRaidTools.db.global.enchantList[spell] ~= nil)) then
		MorthalinRaidTools.db.global.enchants[caster] = {name = spell, expTime = GetServerTime() + (30 * 60)};
	elseif(event == "SPELL_CAST_SUCCESS" and MorthalinRaidTools.db.global.uniqueBuffs[spell] ~= nil and MorthalinRaidTools.db.global.bossfight) then
		if(MorthalinRaidTools.db.global.members[caster].buffs[spell] == nil) then
			MorthalinRaidTools.db.global.members[caster].buffs[spell] = {};
			local _,_,icon = GetSpellInfo(MorthalinRaidTools.db.global.uniqueBuffs[spell].spellId);
			if(MorthalinRaidTools.db.global.uniqueBuffs[spell].icon ~= nil) then
				icon = MorthalinRaidTools.db.global.uniqueBuffs[spell].icon;
			end
			table.insert(MorthalinRaidTools.db.global.members[caster].buffs[spell], {icon = icon, spellId = MorthalinRaidTools.db.global.uniqueBuffs[spell].spellId, count = 1});
		else
			local found = false;
			for i = 1, #MorthalinRaidTools.db.global.members[caster].buffs[spell] do
				if(MorthalinRaidTools.db.global.members[caster].buffs[spell][i].spellId == MorthalinRaidTools.db.global.uniqueBuffs[spell].spellId) then
					MorthalinRaidTools.db.global.members[caster].buffs[spell][i].count = MorthalinRaidTools.db.global.members[caster].buffs[spell][i].count + 1;
					found = true;
				end
			end
			if(found == false) then
				local _,_,icon = GetSpellInfo(MorthalinRaidTools.db.global.uniqueBuffs[spell].spellId);
				if(MorthalinRaidTools.db.global.uniqueBuffs[spell].icon ~= nil) then
					icon = MorthalinRaidTools.db.global.uniqueBuffs[spell].icon;
				end
				table.insert(MorthalinRaidTools.db.global.members[caster].buffs[spell], {icon = icon, spellId = MorthalinRaidTools.db.global.uniqueBuffs[spell].spellId, count = 1});
			end
		end
	end
end

function MorthalinRaidTools:CheckVersion()
	if(GetAddOnMetadata("MorthalinRaidTools", "Version") ~= nil) then
		if(self.db.global.version ~= GetAddOnMetadata("MorthalinRaidTools", "Version")) then
			if(tonumber(self.db.global.version) < 1.31) then
				MorthalinRaidTools:ReconstructBuffs();
			end
			self:Print("Version of addon changed.");
			self.db.global.version = GetAddOnMetadata("MorthalinRaidTools", "Version");
		end
	end
	MorthalinRaidTools:SendBroadcastVersionCheck();
end